package cz.muni.fi.pv168.project.data.category;

import cz.muni.fi.pv168.project.data.Identifiable;
import org.joda.time.Duration;
import org.joda.time.PeriodType;
import org.joda.time.format.PeriodFormatter;
import org.joda.time.format.PeriodFormatterBuilder;

import java.awt.Color;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

public class Category implements Identifiable {

    private Long id;
    private String name;
    private Color color;
    private List<CategoryTime> timeSpents;

    private final PeriodFormatter periodicFormatter;

    public Category(String name, Color color) {
        this.timeSpents = new ArrayList<>();
        this.name = name;
        this.color = color;
        this.periodicFormatter = createFormatter();
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Color getColor() {
        return color;
    }

    public void setColor(Color color) {
        this.color = color;
    }

    @Override
    public String toString() {
        return this.name;
    }

    public String getTimeDurationFormatted(LocalDate fromDate, LocalDate toDate) {
        final var dayTimePeriod = getTotalDuration(fromDate, toDate).toPeriod().normalizedStandard(PeriodType.dayTime());
        return periodicFormatter.print(dayTimePeriod).strip();
    }

    public Duration getTotalDuration(LocalDate fromDate, LocalDate toDate) {
        return Duration.standardHours(this.timeSpents
                .stream()
                .filter(categoryTime -> categoryTime.getCompletionDate().compareTo(fromDate) >= 0)
                .filter(categoryTime -> categoryTime.getCompletionDate().compareTo(toDate) <= 0)
                .map(CategoryTime::getTimeInHours)
                .reduce(0L, Long::sum));
    }

    public void setTimeSpents(List<CategoryTime> timeSpents) {
        this.timeSpents = timeSpents;
    }

    public List<CategoryTime> getTimeSpents() {
        return timeSpents;
    }

    public void addTime(Long hours, Long taskId) {
        timeSpents.add(new CategoryTime(LocalDate.now(), hours, taskId));
    }

    public void subtractTime(Long taskId) {
        timeSpents.removeIf(categoryTime -> categoryTime.getTaskId().equals(taskId));
    }

    //TODO: add support for language structures in localization
    private PeriodFormatter createFormatter() {
        return new PeriodFormatterBuilder()
                .appendDays()
                .appendSuffix(" days ")
                .appendHours()
                .appendSuffix(" hours")
                .toFormatter();
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        Category category = (Category) o;
        return Objects.equals(id, category.id);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }
}
