package cz.muni.fi.pv168.project.model.db;

import cz.muni.fi.pv168.project.model.EditableModel;
import cz.muni.fi.pv168.project.ui.dialog.error.ErrorDialog;
import cz.muni.fi.pv168.project.ui.i18n.I18N;

import javax.swing.DefaultListModel;

public abstract class AbstractDataModel<E> extends DefaultListModel<E> implements EditableModel<E> {

    private static final I18N I18N = new I18N(AbstractDataModel.class);

    @Override
    public void setElementAt(E element, int index) {
        try {
            update(element);
        } catch (Exception e) {
            e.printStackTrace();
            ErrorDialog.show(I18N.getString("editingEntError") + " " + element, e);
        }
        super.setElementAt(element, index);
    }

    @Override
    public void addElement(E element) {
        try {
            add(element);
        } catch (Exception e) {
            e.printStackTrace();
            ErrorDialog.show(I18N.getString("addingEntError") + " " + element, e);
        }

        super.addElement(element);
    }

    @Override
    @SuppressWarnings("unchecked") // safe
    public boolean removeElement(Object obj) {
        try {
            delete((E) obj);
        } catch (Exception e) {
            e.printStackTrace();
            ErrorDialog.show(I18N.getString("deletingEntError") + " " + obj, e);
        }
        return super.removeElement(obj);
    }
}
